@@ -4,6 +4,9 @@
 
 // Test ensures zombie debug lines do not persist
 // https://github.com/firefox-devtools/debugger/issues/7755
+
+"use strict";
+
 add_task(async function() {
   // Load test files
   const dbg = await initDebugger("doc-sources.html");
@@ -16,13 +19,13 @@ add_task(async function() {
   // Trigger the breakpoint ane ensure we're paused
   invokeInTab("main");
   await waitForPaused(dbg);
-  await waitForDispatch(dbg, "ADD_INLINE_PREVIEW");
+  await waitForDispatch(dbg.store, "ADD_INLINE_PREVIEW");
 
   // Scroll element into view
   findElement(dbg, "frame", 2).focus();
 
   // Click the call stack to get to debugger-line-1
-  const dispatched = waitForDispatch(dbg, "ADD_INLINE_PREVIEW");
+  const dispatched = waitForDispatch(dbg.store, "ADD_INLINE_PREVIEW");
   await clickElement(dbg, "frame", 2);
   await dispatched;
   await waitForRequestsToSettle(dbg);
