@@ -6,7 +6,9 @@
 
 const { l10n } = require("devtools/shared/inspector/css-logic");
 const { PSEUDO_CLASSES } = require("devtools/shared/css/constants");
-const { ELEMENT_STYLE } = require("devtools/shared/specs/styles");
+const {
+  style: { ELEMENT_STYLE },
+} = require("devtools/shared/constants");
 const Rule = require("devtools/client/inspector/rules/models/rule");
 const {
   InplaceEditor,
@@ -26,18 +28,11 @@ const {
   SELECTOR_ELEMENT,
   SELECTOR_PSEUDO_CLASS,
 } = require("devtools/shared/css/parsing-utils");
-const promise = require("promise");
 const Services = require("Services");
 const EventEmitter = require("devtools/shared/event-emitter");
 const CssLogic = require("devtools/shared/inspector/css-logic");
 
 loader.lazyRequireGetter(this, "Tools", "devtools/client/definitions", true);
-loader.lazyRequireGetter(
-  this,
-  "gDevTools",
-  "devtools/client/framework/devtools",
-  true
-);
 
 const STYLE_INSPECTOR_PROPERTIES =
   "devtools/shared/locales/styleinspector.properties";
@@ -91,28 +86,8 @@ RuleEditor.prototype = {
     this.toolbox.off("tool-registered", this._onToolChanged);
     this.toolbox.off("tool-unregistered", this._onToolChanged);
 
-    let url = null;
-    if (this.rule.sheet) {
-      url = this.rule.sheet.href || this.rule.sheet.nodeHref;
-    }
-    if (
-      url &&
-      !this.rule.isSystem &&
-      this.rule.domRule.type !== ELEMENT_STYLE
-    ) {
-      // Only get the original source link if the rule isn't a system
-      // rule and if it isn't an inline rule.
-      const sourceLine = this.rule.ruleLine;
-      const sourceColumn = this.rule.ruleColumn;
-
-      if (this._sourceMapURLService) {
-        this._sourceMapURLService.unsubscribe(
-          url,
-          sourceLine,
-          sourceColumn,
-          this._updateLocation
-        );
-      }
+    if (this._unsubscribeSourceMap) {
+      this._unsubscribeSourceMap();
     }
   },
 
@@ -160,6 +135,38 @@ RuleEditor.prototype = {
 
     this.updateSourceLink();
 
+    if (this.rule.domRule.ancestorData.length > 0) {
+      const parts = this.rule.domRule.ancestorData.map(ancestorData => {
+        if (ancestorData.type == "layer") {
+          return `@layer${ancestorData.value ? " " + ancestorData.value : ""}`;
+        }
+        if (ancestorData.type == "media") {
+          return `@media ${ancestorData.value}`;
+        }
+        // We shouldn't get here as `type` can only be set to "layer" or "media", but just
+        // in case, let's return an empty string.
+        console.warn("Unknown ancestor data type:", ancestorData.type);
+        return ``;
+      });
+
+      // We force the string to be LTR in CSS, but as @ is listed as having neutral
+      // directionality and starting a string with this char would default to RTL for that
+      // character (when in RTL locale), and then the next char (`m` of `media`, or `l` of `layer`)
+      // would start a new LTR visual run, since it is strongly LTR (through `direction` CSS property).
+      // To have the `@` properly displayed, we force LTR with \u202A
+      const title = `${parts.join("\n").replaceAll("@", "\u202A@")}`;
+
+      this.ancestorDataEl = createChild(this.element, "ul", {
+        class: "ruleview-rule-ancestor-data theme-link",
+        title,
+      });
+      for (const part of parts) {
+        createChild(this.ancestorDataEl, "li", {
+          textContent: part,
+        });
+      }
+    }
+
     const code = createChild(this.element, "div", {
       class: "ruleview-code",
     });
@@ -185,6 +192,7 @@ RuleEditor.prototype = {
     }
 
     if (this.rule.domRule.type !== CSSRule.KEYFRAME_RULE) {
+      // FIXME: Avoid having this as a nested async operation. (Bug 1664511)
       (async function() {
         let selector;
 
@@ -197,29 +205,18 @@ RuleEditor.prototype = {
           selector = await this.rule.inherited.getUniqueSelector();
         } else {
           // This is an inline style from the current node.
-          selector = this.ruleView.inspector.selectionCssSelector;
+          selector = await this.ruleView.inspector.selection.nodeFront.getUniqueSelector();
         }
 
-        const isHighlighted =
-          this.ruleView._highlighters &&
-          this.ruleView.highlighters.selectorHighlighterShown === selector;
-        const selectorHighlighter = createChild(header, "span", {
+        const isHighlighted = this.ruleView.isSelectorHighlighted(selector);
+        // Handling of click events is delegated to CssRuleView.handleEvent()
+        createChild(header, "span", {
           class:
-            "ruleview-selectorhighlighter" +
+            "ruleview-selectorhighlighter js-toggle-selector-highlighter" +
             (isHighlighted ? " highlighted" : ""),
+          "data-selector": selector,
           title: l10n("rule.selectorHighlighter.tooltip"),
         });
-        selectorHighlighter.addEventListener("click", event => {
-          this.ruleView.toggleSelectorHighlighter(
-            selectorHighlighter,
-            selector
-          );
-          // Prevent clicks from focusing the property editor.
-          event.stopPropagation();
-        });
-
-        this.uniqueSelector = selector;
-        this.emit("selector-icon-created");
       }
         .bind(this)()
         .catch(error => {
@@ -300,72 +297,46 @@ RuleEditor.prototype = {
   },
 
   _onSourceClick: function() {
-    if (this.source.hasAttribute("unselectable") || !this._currentLocation) {
+    if (this.source.hasAttribute("unselectable")) {
       return;
     }
 
-    const target = this.ruleView.inspector.currentTarget;
-    if (Tools.styleEditor.isTargetSupported(target)) {
-      gDevTools.showToolbox(target, "styleeditor").then(toolbox => {
-        const { url, line, column } = this._currentLocation;
-
-        if (!this.rule.sheet.href && this.rule.sheet.nodeHref) {
-          toolbox
-            .getCurrentPanel()
-            .selectStyleSheet(this.rule.sheet, line, column);
-        } else {
-          toolbox.getCurrentPanel().selectStyleSheet(url, line, column);
-        }
-      });
+    const { inspector } = this.ruleView;
+    if (Tools.styleEditor.isToolSupported(inspector.toolbox)) {
+      inspector.toolbox.viewSourceInStyleEditorByFront(
+        this.rule.sheet,
+        this.rule.ruleLine,
+        this.rule.ruleColumn
+      );
     }
   },
 
   /**
    * Update the text of the source link to reflect whether we're showing
    * original sources or not.  This is a callback for
-   * SourceMapURLService.subscribe, which see.
+   * SourceMapURLService.subscribeByID, which see.
    *
-   * @param {Boolean} enabled
-   *        True if the passed-in location should be used; this means
-   *        that source mapping is in use and the remaining arguments
-   *        are the original location.  False if the already-known
-   *        (stored) location should be used.
-   * @param {String} url
-   *        The original URL
-   * @param {Number} line
-   *        The original line number
-   * @param {number} column
-   *        The original column number
+   * @param {Object | null} originalLocation
+   *        The original position object (url/line/column) or null.
    */
-  _updateLocation: function(enabled, url, line, column) {
-    let displayURL = url;
-    if (!enabled) {
-      url = null;
-      displayURL = null;
-      if (this.rule.sheet) {
-        url = this.rule.sheet.href || this.rule.sheet.nodeHref;
-        displayURL = this.rule.sheet.href;
-      }
-      line = this.rule.ruleLine;
-      column = this.rule.ruleColumn;
+  _updateLocation: function(originalLocation) {
+    let displayURL = this.rule.sheet?.href;
+    const constructed = this.rule.sheet?.constructed;
+    let line = this.rule.ruleLine;
+    if (originalLocation) {
+      displayURL = originalLocation.url;
+      line = originalLocation.line;
     }
 
-    this._currentLocation = {
-      url,
-      line,
-      column,
-    };
-
-    let sourceTextContent = CssLogic.shortSource({ href: displayURL });
+    let sourceTextContent = CssLogic.shortSource({
+      constructed,
+      href: displayURL,
+    });
     let title = displayURL ? displayURL : sourceTextContent;
     if (line > 0) {
       sourceTextContent += ":" + line;
       title += ":" + line;
     }
-    if (this.rule.mediaText) {
-      sourceTextContent += " @" + this.rule.mediaText;
-      title += " @" + this.rule.mediaText;
-    }
 
     const sourceLabel = this.element.querySelector(
       ".ruleview-rule-source-label"
@@ -380,8 +351,7 @@ RuleEditor.prototype = {
         ".ruleview-rule-source-label"
       );
       const title = this.rule.title;
-      const sourceHref =
-        this.rule.sheet && this.rule.sheet.href ? this.rule.sheet.href : title;
+      const sourceHref = this.rule.sheet?.href || title;
 
       const uaLabel = STYLE_INSPECTOR_L10N.getStr("rule.userAgentStyles");
       sourceLabel.textContent = uaLabel + " " + title;
@@ -395,26 +365,23 @@ RuleEditor.prototype = {
         sourceLabel.removeAttribute("title");
       }
     } else {
-      this._updateLocation(false);
+      this._updateLocation(null);
     }
 
-    let url = null;
-    if (this.rule.sheet) {
-      url = this.rule.sheet.href || this.rule.sheet.nodeHref;
-    }
     if (
-      url &&
+      this.rule.sheet &&
       !this.rule.isSystem &&
       this.rule.domRule.type !== ELEMENT_STYLE
     ) {
       // Only get the original source link if the rule isn't a system
       // rule and if it isn't an inline rule.
-      const sourceLine = this.rule.ruleLine;
-      const sourceColumn = this.rule.ruleColumn;
-      this.sourceMapURLService.subscribe(
-        url,
-        sourceLine,
-        sourceColumn,
+      if (this._unsubscribeSourceMap) {
+        this._unsubscribeSourceMap();
+      }
+      this._unsubscribeSourceMap = this.sourceMapURLService.subscribeByID(
+        this.rule.sheet.resourceId || this.rule.sheet.actorID,
+        this.rule.ruleLine,
+        this.rule.ruleColumn,
         this._updateLocation
       );
       // Set "unselectable" appropriately.
@@ -426,7 +393,7 @@ RuleEditor.prototype = {
       this._onToolChanged();
     }
 
-    promise.resolve().then(() => {
+    Promise.resolve().then(() => {
       this.emit("source-link-updated");
     });
   },
@@ -722,6 +689,11 @@ RuleEditor.prototype = {
 
     this.isEditing = true;
 
+    // Remove highlighter for the previous selector.
+    if (this.ruleView.isSelectorHighlighted(this.rule.selectorText)) {
+      await this.ruleView.toggleSelectorHighlighter(this.rule.selectorText);
+    }
+
     try {
       const response = await this.rule.domRule.modifySelector(element, value);
 
@@ -769,14 +741,6 @@ RuleEditor.prototype = {
       // pseudo-element rules and the like.
       this.element.parentNode.replaceChild(editor.element, this.element);
 
-      // Remove highlight for modified selector
-      if (ruleView.highlighters.selectorHighlighterShown) {
-        ruleView.toggleSelectorHighlighter(
-          ruleView.lastSelectorIcon,
-          ruleView.highlighters.selectorHighlighterShown
-        );
-      }
-
       editor._moveSelectorFocus(direction);
     } catch (err) {
       this.isEditing = false;
