@@ -3,32 +3,45 @@
 
 "use strict";
 
+XPCOMUtils.defineLazyGetter(this, "jsonViewStrings", () => {
+  return Services.strings.createBundle(
+    "chrome://devtools/locale/jsonview.properties"
+  );
+});
+
 const TEST_JSON_URL = URL_ROOT + "array_json.json";
 const EXPAND_THRESHOLD = 100 * 1024;
 
 add_task(async function() {
-  info("Test expand/collapse JSON started");
+  info("Test expand/collapse small JSON started");
 
   await addJsonViewTab(TEST_JSON_URL);
-  const browser = gBrowser.selectedBrowser;
 
   /* Initial sanity check */
   const countBefore = await getElementCount(".treeRow");
-  ok(countBefore == 6, "There must be six rows");
+  is(countBefore, 6, "There must be six rows");
 
   /* Test the "Collapse All" button */
   let selector = ".jsonPanelBox .toolbar button.collapse";
-  await BrowserTestUtils.synthesizeMouseAtCenter(selector, {}, browser);
+  await clickJsonNode(selector);
   let countAfter = await getElementCount(".treeRow");
-  ok(countAfter == 3, "There must be three rows");
+  is(countAfter, 3, "There must be three rows");
 
   /* Test the "Expand All" button */
   selector = ".jsonPanelBox .toolbar button.expand";
-  await BrowserTestUtils.synthesizeMouseAtCenter(selector, {}, browser);
+  is(
+    await getElementText(selector),
+    jsonViewStrings.GetStringFromName("jsonViewer.ExpandAll"),
+    "Expand button doesn't warn that the action will be slow"
+  );
+  await clickJsonNode(selector);
   countAfter = await getElementCount(".treeRow");
-  ok(countAfter == 6, "There must be six expanded rows");
+  is(countAfter, 6, "There must be six expanded rows");
+});
+
+add_task(async function() {
+  info("Test expand button for big JSON started");
 
-  /* Test big file handling */
   const json = JSON.stringify({
     data: Array(1e5)
       .fill()
@@ -40,8 +53,9 @@ add_task(async function() {
     "The generated JSON must be larger than 100kB"
   );
   await addJsonViewTab("data:application/json," + json);
-  ok(
-    document.querySelector(selector) == null,
-    "The Expand All button must be gone"
+  is(
+    await getElementText(".jsonPanelBox .toolbar button.expand"),
+    jsonViewStrings.GetStringFromName("jsonViewer.ExpandAllSlow"),
+    "Expand button warns that the action will be slow"
   );
 });
