@@ -4,6 +4,7 @@
 
 "use strict";
 
+const Services = require("Services");
 const {
   Component,
   createFactory,
@@ -17,12 +18,14 @@ const dom = require("devtools/client/shared/vendor/react-dom-factories");
 const {
   getFormattedIPAndPort,
   getFormattedSize,
+  getRequestPriorityAsText,
 } = require("devtools/client/netmonitor/src/utils/format-utils");
 const { L10N } = require("devtools/client/netmonitor/src/utils/l10n");
 const {
   getHeadersURL,
   getTrackingProtectionURL,
-} = require("devtools/client/netmonitor/src/utils/mdn-utils");
+  getHTTPStatusCodeURL,
+} = require("devtools/client/netmonitor/src/utils/doc-utils");
 const {
   fetchNetworkUpdatePacket,
   writeHeaderText,
@@ -47,6 +50,7 @@ const Accordion = createFactory(
 const UrlPreview = createFactory(
   require("devtools/client/netmonitor/src/components/previews/UrlPreview")
 );
+const HeadersPanelContextMenu = require("devtools/client/netmonitor/src/widgets/HeadersPanelContextMenu");
 const StatusCode = createFactory(
   require("devtools/client/netmonitor/src/components/StatusCode")
 );
@@ -55,10 +59,10 @@ loader.lazyGetter(this, "MDNLink", function() {
   return createFactory(require("devtools/client/shared/components/MdnLink"));
 });
 loader.lazyGetter(this, "Rep", function() {
-  return require("devtools/client/shared/components/reps/reps").REPS.Rep;
+  return require("devtools/client/shared/components/reps/index").REPS.Rep;
 });
 loader.lazyGetter(this, "MODE", function() {
-  return require("devtools/client/shared/components/reps/reps").MODE;
+  return require("devtools/client/shared/components/reps/index").MODE;
 });
 loader.lazyGetter(this, "TreeRow", function() {
   return createFactory(
@@ -71,6 +75,12 @@ loader.lazyRequireGetter(
   "devtools/client/shared/components/menu/utils",
   true
 );
+loader.lazyRequireGetter(
+  this,
+  "openContentLink",
+  "devtools/client/shared/link",
+  true
+);
 
 const { div, input, label, span, textarea, tr, td, button } = dom;
 
@@ -86,6 +96,9 @@ const HEADERS_STATUS = L10N.getStr("netmonitor.headers.status");
 const HEADERS_VERSION = L10N.getStr("netmonitor.headers.version");
 const HEADERS_TRANSFERRED = L10N.getStr("netmonitor.toolbar.transferred");
 const SUMMARY_STATUS_LEARN_MORE = L10N.getStr("netmonitor.summary.learnMore");
+const SUMMARY_ETP_LEARN_MORE = L10N.getStr(
+  "netmonitor.enhancedTrackingProtection.learnMore"
+);
 const HEADERS_REFERRER = L10N.getStr("netmonitor.headers.referrerPolicy");
 const HEADERS_CONTENT_BLOCKING = L10N.getStr(
   "netmonitor.headers.contentBlocking"
@@ -93,13 +106,14 @@ const HEADERS_CONTENT_BLOCKING = L10N.getStr(
 const HEADERS_ETP = L10N.getStr(
   "netmonitor.trackingResource.enhancedTrackingProtection"
 );
+const HEADERS_PRIORITY = L10N.getStr("netmonitor.headers.requestPriority");
 
 /**
  * Headers panel component
  * Lists basic information about the request
  *
  * In http/2 all response headers are in small case.
- * See: https://developer.mozilla.org/en-US/docs/Tools/Network_Monitor/request_details#Headers
+ * See: https://firefox-source-docs.mozilla.org/devtools-user/network_monitor/request_details/index.html#response-headers
  * RFC: https://tools.ietf.org/html/rfc7540#section-8.1.2
  */
 class HeadersPanel extends Component {
@@ -113,8 +127,11 @@ class HeadersPanel extends Component {
       openLink: PropTypes.func,
       targetSearchResult: PropTypes.object,
       openRequestBlockingAndAddUrl: PropTypes.func.isRequired,
+      openHTTPCustomRequestTab: PropTypes.func.isRequired,
       cloneRequest: PropTypes.func,
       sendCustomRequest: PropTypes.func,
+      shouldExpandPreview: PropTypes.bool,
+      setHeadersUrlPreviewExpanded: PropTypes.func,
     };
   }
 
@@ -139,6 +156,7 @@ class HeadersPanel extends Component {
     this.renderValue = this.renderValue.bind(this);
     this.renderRawHeadersBtn = this.renderRawHeadersBtn.bind(this);
     this.onShowResendMenu = this.onShowResendMenu.bind(this);
+    this.onShowHeadersContextMenu = this.onShowHeadersContextMenu.bind(this);
   }
 
   componentDidMount() {
@@ -180,11 +198,14 @@ class HeadersPanel extends Component {
             3
           )})`;
         } else {
-          preHeaderText = `${method} ${
-            urlDetails.url.split(
-              requestHeaders.headers.find(ele => ele.name === "Host").value
-            )[1]
-          } ${httpVersion}`;
+          const hostHeader = requestHeaders.headers.find(
+            ele => ele.name === "Host"
+          );
+          if (hostHeader) {
+            preHeaderText = `${method} ${
+              urlDetails.url.split(hostHeader.value)[1]
+            } ${httpVersion}`;
+          }
           result = `${title} (${getFormattedSize(
             writeHeaderText(requestHeaders.headers, preHeaderText).length,
             3
@@ -345,10 +366,11 @@ class HeadersPanel extends Component {
       const rawHeaderType = this.getRawHeaderType(path);
       switch (rawHeaderType) {
         case "REQUEST":
+          const hostHeader = requestHeaders.headers.find(
+            ele => ele.name === "Host"
+          );
           preHeaderText = `${method} ${
-            urlDetails.url.split(
-              requestHeaders.headers.find(ele => ele.name === "Host").value
-            )[1]
+            hostHeader ? urlDetails.url.split(hostHeader.value)[1] : "<unknown>"
           } ${httpVersion}`;
           value = writeHeaderText(requestHeaders.headers, preHeaderText).trim();
           break;
@@ -366,9 +388,11 @@ class HeadersPanel extends Component {
 
       let rows;
       if (value) {
+        const match = value.match(/\n/g);
+        rows = match !== null ? match.length : 0;
         // Need to add 1 for the horizontal scrollbar
         // not to cover the last row of raw data
-        rows = value.match(/\n/g).length + 1;
+        rows = rows + 1;
       }
 
       return tr(
@@ -443,6 +467,7 @@ class HeadersPanel extends Component {
           member: Object.assign({}, member, { open: false }),
           mode: MODE.TINY,
           noGrip: true,
+          openLink: openContentLink,
         })
       ),
       headerDocURL ? MDNLink({ url: headerDocURL }) : null
@@ -494,6 +519,13 @@ class HeadersPanel extends Component {
     showMenu(menuItems, { button: event.target });
   }
 
+  onShowHeadersContextMenu(event) {
+    if (!this.contextMenu) {
+      this.contextMenu = new HeadersPanelContextMenu();
+    }
+    this.contextMenu.open(event, window.getSelection());
+  }
+
   render() {
     const {
       targetSearchResult,
@@ -511,11 +543,15 @@ class HeadersPanel extends Component {
         statusText,
         urlDetails,
         referrerPolicy,
+        priority,
         isThirdPartyTrackingResource,
         contentSize,
         transferredSize,
       },
       openRequestBlockingAndAddUrl,
+      openHTTPCustomRequestTab,
+      shouldExpandPreview,
+      setHeadersUrlPreviewExpanded,
     } = this.props;
     const {
       rawResponseHeadersOpened,
@@ -662,7 +698,11 @@ class HeadersPanel extends Component {
           StatusCode({
             item: { fromCache, fromServiceWorker, status, statusText },
           }),
-          statusText
+          statusText,
+          MDNLink({
+            url: getHTTPStatusCodeURL(status),
+            title: SUMMARY_STATUS_LEARN_MORE,
+          })
         )
       );
     }
@@ -687,7 +727,7 @@ class HeadersPanel extends Component {
           trackingProtectionDocURL
             ? MDNLink({
                 url: trackingProtectionDocURL,
-                title: SUMMARY_STATUS_LEARN_MORE,
+                title: SUMMARY_ETP_LEARN_MORE,
               })
             : span({ className: "headers-summary learn-more-link" })
         )
@@ -702,15 +742,24 @@ class HeadersPanel extends Component {
       ? this.renderSummary(HEADERS_REFERRER, referrerPolicy)
       : null;
 
+    const summaryPriority = priority
+      ? this.renderSummary(HEADERS_PRIORITY, getRequestPriorityAsText(priority))
+      : null;
+
     const summaryItems = [
       summaryStatus,
       summaryVersion,
       summarySize,
       summaryReferrerPolicy,
+      summaryPriority,
       trackingProtectionStatus,
       trackingProtectionDetails,
     ].filter(summaryItem => summaryItem !== null);
 
+    const newEditAndResendPref = Services.prefs.getBoolPref(
+      "devtools.netmonitor.features.newEditAndResend"
+    );
+
     return div(
       { className: "headers-panel-container" },
       div(
@@ -735,9 +784,15 @@ class HeadersPanel extends Component {
         button(
           {
             id: "edit-resend-button",
-            className: "devtools-button devtools-dropdown-button",
+            className: !newEditAndResendPref
+              ? "devtools-button devtools-dropdown-button"
+              : "devtools-button",
             title: RESEND,
-            onClick: this.onShowResendMenu,
+            onClick: !newEditAndResendPref
+              ? this.onShowResendMenu
+              : () => {
+                  openHTTPCustomRequestTab();
+                },
           },
           span({ className: "title" }, RESEND)
         )
@@ -752,8 +807,16 @@ class HeadersPanel extends Component {
             address: remoteAddress
               ? getFormattedIPAndPort(remoteAddress, remotePort)
               : null,
+            shouldExpandPreview,
+            onTogglePreview: expanded => setHeadersUrlPreviewExpanded(expanded),
           }),
-          div({ className: "summary" }, summaryItems)
+          div(
+            {
+              className: "summary",
+              onContextMenu: this.onShowHeadersContextMenu,
+            },
+            summaryItems
+          )
         ),
         Accordion({ items })
       )
@@ -761,9 +824,19 @@ class HeadersPanel extends Component {
   }
 }
 
-module.exports = connect(null, (dispatch, props) => ({
-  openRequestBlockingAndAddUrl: url =>
-    dispatch(Actions.openRequestBlockingAndAddUrl(url)),
-  cloneRequest: id => dispatch(Actions.cloneRequest(id)),
-  sendCustomRequest: () => dispatch(Actions.sendCustomRequest(props.connector)),
-}))(HeadersPanel);
+module.exports = connect(
+  state => ({
+    shouldExpandPreview: state.ui.shouldExpandHeadersUrlPreview,
+  }),
+  (dispatch, props) => ({
+    setHeadersUrlPreviewExpanded: expanded =>
+      dispatch(Actions.setHeadersUrlPreviewExpanded(expanded)),
+    openRequestBlockingAndAddUrl: url =>
+      dispatch(Actions.openRequestBlockingAndAddUrl(url)),
+    openHTTPCustomRequestTab: () =>
+      dispatch(Actions.openHTTPCustomRequest(true)),
+    cloneRequest: id => dispatch(Actions.cloneRequest(id)),
+    sendCustomRequest: () =>
+      dispatch(Actions.sendCustomRequest(props.connector)),
+  })
+)(HeadersPanel);
