@@ -7,18 +7,13 @@
 "use strict";
 
 const TEST_URI =
-  "http://example.com/browser/devtools/client/webconsole/" +
+  "https://example.com/browser/devtools/client/webconsole/" +
   "test/browser/test-iframe-parent.html";
 
 add_task(async function() {
   const hud = await openNewTabAndConsole(TEST_URI);
 
-  await executeAndWaitForMessage(
-    hud,
-    "document.title",
-    "iframe parent",
-    ".result"
-  );
+  await executeAndWaitForResultMessage(hud, "document.title", "iframe parent");
   ok(true, "root document's title is accessible");
 
   // Make sure we don't throw when trying to autocomplete
@@ -28,7 +23,11 @@ add_task(async function() {
   await autocompleteUpdated;
 
   setInputValue(hud, "window[0].document.title");
-  const onPermissionDeniedMessage = waitForMessage(hud, "Permission denied");
+  const onPermissionDeniedMessage = waitForMessageByType(
+    hud,
+    "Permission denied",
+    ".error"
+  );
   EventUtils.synthesizeKey("KEY_Enter");
   const permissionDenied = await onPermissionDeniedMessage;
   ok(
@@ -36,11 +35,10 @@ add_task(async function() {
     "A message error is shown when trying to inspect window[0]"
   );
 
-  await executeAndWaitForMessage(
+  await executeAndWaitForResultMessage(
     hud,
     "window.location",
-    "test-iframe-parent.html",
-    ".result"
+    "test-iframe-parent.html"
   );
   ok(true, "root document's location is accessible");
 });
