@@ -4,10 +4,8 @@
 
 "use strict";
 
-const { Cc, Ci } = require("chrome");
-const { generateUUID } = Cc["@mozilla.org/uuid-generator;1"].getService(
-  Ci.nsIUUIDGenerator
-);
+const Services = require("Services");
+const { generateUUID } = Services.uuid;
 
 /**
  * Returns a new `uuid`.
