@@ -14,8 +14,9 @@
 #include "nsContentCreatorFunctions.h"
 #include "nsCSSPseudoElements.h"
 #include "nsCSSRendering.h"
-#include "nsCheckboxRadioFrame.h"
+#include "nsDisplayList.h"
 #include "nsIContent.h"
+#include "nsLayoutUtils.h"
 #include "mozilla/dom/Document.h"
 #include "nsNameSpaceManager.h"
 #include "nsGkAtoms.h"
@@ -39,8 +40,6 @@ using namespace mozilla;
 using namespace mozilla::dom;
 using namespace mozilla::image;
 
-NS_IMPL_ISUPPORTS(nsRangeFrame::DummyTouchListener, nsIDOMEventListener)
-
 nsIFrame* NS_NewRangeFrame(PresShell* aPresShell, ComputedStyle* aStyle) {
   return new (aPresShell) nsRangeFrame(aStyle, aPresShell->GetPresContext());
 }
@@ -48,7 +47,7 @@ nsIFrame* NS_NewRangeFrame(PresShell* aPresShell, ComputedStyle* aStyle) {
 nsRangeFrame::nsRangeFrame(ComputedStyle* aStyle, nsPresContext* aPresContext)
     : nsContainerFrame(aStyle, aPresContext, kClassID) {}
 
-nsRangeFrame::~nsRangeFrame() {}
+nsRangeFrame::~nsRangeFrame() = default;
 
 NS_IMPL_FRAMEARENA_HELPERS(nsRangeFrame)
 
@@ -57,39 +56,12 @@ NS_QUERYFRAME_HEAD(nsRangeFrame)
   NS_QUERYFRAME_ENTRY(nsIAnonymousContentCreator)
 NS_QUERYFRAME_TAIL_INHERITING(nsContainerFrame)
 
-void nsRangeFrame::Init(nsIContent* aContent, nsContainerFrame* aParent,
-                        nsIFrame* aPrevInFlow) {
-  // With APZ enabled, touch events may be handled directly by the APZC code
-  // if the APZ knows that there is no content interested in the touch event.
-  // The range input element *is* interested in touch events, but doesn't use
-  // the usual mechanism (i.e. registering an event listener) to handle touch
-  // input. Instead, we do it here so that the APZ finds out about it, and
-  // makes sure to wait for content to run handlers before handling the touch
-  // input itself.
-  if (!mDummyTouchListener) {
-    mDummyTouchListener = new DummyTouchListener();
-  }
-  aContent->AddEventListener(NS_LITERAL_STRING("touchstart"),
-                             mDummyTouchListener, false);
-
-  ServoStyleSet* styleSet = PresContext()->StyleSet();
-
-  mOuterFocusStyle = styleSet->ProbePseudoElementStyle(
-      *aContent->AsElement(), PseudoStyleType::mozFocusOuter, Style());
-
-  return nsContainerFrame::Init(aContent, aParent, aPrevInFlow);
-}
-
 void nsRangeFrame::DestroyFrom(nsIFrame* aDestructRoot,
                                PostDestroyData& aPostDestroyData) {
   NS_ASSERTION(!GetPrevContinuation() && !GetNextContinuation(),
                "nsRangeFrame should not have continuations; if it does we "
                "need to call RegUnregAccessKey only for the first.");
 
-  mContent->RemoveEventListener(NS_LITERAL_STRING("touchstart"),
-                                mDummyTouchListener, false);
-
-  nsCheckboxRadioFrame::RegUnRegAccessKey(static_cast<nsIFrame*>(this), false);
   aPostDestroyData.AddAnonymousContent(mTrackDiv.forget());
   aPostDestroyData.AddAnonymousContent(mProgressDiv.forget());
   aPostDestroyData.AddAnonymousContent(mThumbDiv.forget());
@@ -105,9 +77,9 @@ nsresult nsRangeFrame::MakeAnonymousDiv(Element** aResult,
   // Associate the pseudo-element with the anonymous child.
   resultElement->SetPseudoElementType(aPseudoType);
 
-  if (!aElements.AppendElement(resultElement)) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
+  // XXX(Bug 1631371) Check if this should use a fallible operation as it
+  // pretended earlier, or change the return type to void.
+  aElements.AppendElement(resultElement);
 
   resultElement.forget(aResult);
   return NS_OK;
@@ -148,82 +120,6 @@ void nsRangeFrame::AppendAnonymousContentTo(nsTArray<nsIContent*>& aElements,
   }
 }
 
-class nsDisplayRangeFocusRing final : public nsPaintedDisplayItem {
- public:
-  nsDisplayRangeFocusRing(nsDisplayListBuilder* aBuilder, nsIFrame* aFrame)
-      : nsPaintedDisplayItem(aBuilder, aFrame) {
-    MOZ_COUNT_CTOR(nsDisplayRangeFocusRing);
-  }
-#ifdef NS_BUILD_REFCNT_LOGGING
-  virtual ~nsDisplayRangeFocusRing() {
-    MOZ_COUNT_DTOR(nsDisplayRangeFocusRing);
-  }
-#endif
-
-  nsDisplayItemGeometry* AllocateGeometry(
-      nsDisplayListBuilder* aBuilder) override;
-  void ComputeInvalidationRegion(nsDisplayListBuilder* aBuilder,
-                                 const nsDisplayItemGeometry* aGeometry,
-                                 nsRegion* aInvalidRegion) const override;
-  virtual nsRect GetBounds(nsDisplayListBuilder* aBuilder,
-                           bool* aSnap) const override;
-  virtual void Paint(nsDisplayListBuilder* aBuilder, gfxContext* aCtx) override;
-  NS_DISPLAY_DECL_NAME("RangeFocusRing", TYPE_RANGE_FOCUS_RING)
-};
-
-nsDisplayItemGeometry* nsDisplayRangeFocusRing::AllocateGeometry(
-    nsDisplayListBuilder* aBuilder) {
-  return new nsDisplayItemGenericImageGeometry(this, aBuilder);
-}
-
-void nsDisplayRangeFocusRing::ComputeInvalidationRegion(
-    nsDisplayListBuilder* aBuilder, const nsDisplayItemGeometry* aGeometry,
-    nsRegion* aInvalidRegion) const {
-  auto geometry =
-      static_cast<const nsDisplayItemGenericImageGeometry*>(aGeometry);
-
-  if (aBuilder->ShouldSyncDecodeImages() &&
-      geometry->ShouldInvalidateToSyncDecodeImages()) {
-    bool snap;
-    aInvalidRegion->Or(*aInvalidRegion, GetBounds(aBuilder, &snap));
-  }
-
-  nsDisplayItem::ComputeInvalidationRegion(aBuilder, aGeometry, aInvalidRegion);
-}
-
-nsRect nsDisplayRangeFocusRing::GetBounds(nsDisplayListBuilder* aBuilder,
-                                          bool* aSnap) const {
-  *aSnap = false;
-  nsRect rect(ToReferenceFrame(), Frame()->GetSize());
-
-  // We want to paint as if specifying a border for ::-moz-focus-outer
-  // specifies an outline for our frame, so inflate by the border widths:
-  ComputedStyle* computedStyle =
-      static_cast<nsRangeFrame*>(mFrame)->mOuterFocusStyle;
-  MOZ_ASSERT(computedStyle, "We only exist if mOuterFocusStyle is non-null");
-  rect.Inflate(computedStyle->StyleBorder()->GetComputedBorder());
-
-  return rect;
-}
-
-void nsDisplayRangeFocusRing::Paint(nsDisplayListBuilder* aBuilder,
-                                    gfxContext* aCtx) {
-  bool unused;
-  ComputedStyle* computedStyle =
-      static_cast<nsRangeFrame*>(mFrame)->mOuterFocusStyle;
-  MOZ_ASSERT(computedStyle, "We only exist if mOuterFocusStyle is non-null");
-
-  PaintBorderFlags flags = aBuilder->ShouldSyncDecodeImages()
-                               ? PaintBorderFlags::SyncDecodeImages
-                               : PaintBorderFlags();
-
-  ImgDrawResult result = nsCSSRendering::PaintBorder(
-      mFrame->PresContext(), *aCtx, mFrame, GetPaintRect(),
-      GetBounds(aBuilder, &unused), computedStyle, flags);
-
-  nsDisplayItemGenericImageGeometry::UpdateDrawResult(this, result);
-}
-
 void nsRangeFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
                                     const nsDisplayListSet& aLists) {
   const nsStyleDisplay* disp = StyleDisplay();
@@ -239,38 +135,11 @@ void nsRangeFrame::BuildDisplayList(nsDisplayListBuilder* aBuilder,
     nsIFrame* thumb = mThumbDiv->GetPrimaryFrame();
     if (thumb) {
       nsDisplayListSet set(aLists, aLists.Content());
-      BuildDisplayListForChild(aBuilder, thumb, set, DISPLAY_CHILD_INLINE);
+      BuildDisplayListForChild(aBuilder, thumb, set, DisplayChildFlag::Inline);
     }
   } else {
     BuildDisplayListForInline(aBuilder, aLists);
   }
-
-  // Draw a focus outline if appropriate:
-
-  if (!aBuilder->IsForPainting() || !IsVisibleForPainting()) {
-    // we don't want the focus ring item for hit-testing or if the item isn't
-    // in the area being [re]painted
-    return;
-  }
-
-  EventStates eventStates = mContent->AsElement()->State();
-  if (eventStates.HasState(NS_EVENT_STATE_DISABLED) ||
-      !eventStates.HasState(NS_EVENT_STATE_FOCUSRING)) {
-    return;  // can't have focus or doesn't match :-moz-focusring
-  }
-
-  if (!mOuterFocusStyle || !mOuterFocusStyle->StyleBorder()->HasBorder()) {
-    // no ::-moz-focus-outer specified border (how style specifies a focus ring
-    // for range)
-    return;
-  }
-
-  if (IsThemed(disp) &&
-      PresContext()->GetTheme()->ThemeDrawsFocusForWidget(disp->mAppearance)) {
-    return;  // the native theme displays its own visual indication of focus
-  }
-
-  aLists.Content()->AppendNewToTop<nsDisplayRangeFocusRing>(aBuilder, this);
 }
 
 void nsRangeFrame::Reflow(nsPresContext* aPresContext,
@@ -289,21 +158,15 @@ void nsRangeFrame::Reflow(nsPresContext* aPresContext,
                "nsRangeFrame should not have continuations; if it does we "
                "need to call RegUnregAccessKey only for the first.");
 
-  if (mState & NS_FRAME_FIRST_REFLOW) {
-    nsCheckboxRadioFrame::RegUnRegAccessKey(this, true);
-  }
-
   WritingMode wm = aReflowInput.GetWritingMode();
   nscoord computedBSize = aReflowInput.ComputedBSize();
-  if (computedBSize == NS_AUTOHEIGHT) {
+  if (computedBSize == NS_UNCONSTRAINEDSIZE) {
     computedBSize = 0;
   }
+  const auto borderPadding = aReflowInput.ComputedLogicalBorderPadding(wm);
   LogicalSize finalSize(
-      wm,
-      aReflowInput.ComputedISize() +
-          aReflowInput.ComputedLogicalBorderPadding().IStartEnd(wm),
-      computedBSize +
-          aReflowInput.ComputedLogicalBorderPadding().BStartEnd(wm));
+      wm, aReflowInput.ComputedISize() + borderPadding.IStartEnd(wm),
+      computedBSize + borderPadding.BStartEnd(wm));
   aDesiredSize.SetSize(wm, finalSize);
 
   ReflowAnonymousContent(aPresContext, aDesiredSize, aReflowInput);
@@ -339,7 +202,7 @@ void nsRangeFrame::ReflowAnonymousContent(nsPresContext* aPresContext,
   // for our anonymous content:
   nscoord rangeFrameContentBoxWidth = aReflowInput.ComputedWidth();
   nscoord rangeFrameContentBoxHeight = aReflowInput.ComputedHeight();
-  if (rangeFrameContentBoxHeight == NS_AUTOHEIGHT) {
+  if (rangeFrameContentBoxHeight == NS_UNCONSTRAINEDSIZE) {
     rangeFrameContentBoxHeight = 0;
   }
 
@@ -379,12 +242,13 @@ void nsRangeFrame::ReflowAnonymousContent(nsPresContext* aPresContext,
     nsReflowStatus frameStatus;
     ReflowOutput trackDesiredSize(aReflowInput);
     ReflowChild(trackFrame, aPresContext, trackDesiredSize, trackReflowInput,
-                trackX, trackY, 0, frameStatus);
+                trackX, trackY, ReflowChildFlags::Default, frameStatus);
     MOZ_ASSERT(
         frameStatus.IsFullyComplete(),
         "We gave our child unconstrained height, so it should be complete");
     FinishReflowChild(trackFrame, aPresContext, trackDesiredSize,
-                      &trackReflowInput, trackX, trackY, 0);
+                      &trackReflowInput, trackX, trackY,
+                      ReflowChildFlags::Default);
   }
 
   nsIFrame* thumbFrame = mThumbDiv->GetPrimaryFrame();
@@ -402,12 +266,12 @@ void nsRangeFrame::ReflowAnonymousContent(nsPresContext* aPresContext,
     nsReflowStatus frameStatus;
     ReflowOutput thumbDesiredSize(aReflowInput);
     ReflowChild(thumbFrame, aPresContext, thumbDesiredSize, thumbReflowInput, 0,
-                0, 0, frameStatus);
+                0, ReflowChildFlags::Default, frameStatus);
     MOZ_ASSERT(
         frameStatus.IsFullyComplete(),
         "We gave our child unconstrained height, so it should be complete");
     FinishReflowChild(thumbFrame, aPresContext, thumbDesiredSize,
-                      &thumbReflowInput, 0, 0, 0);
+                      &thumbReflowInput, 0, 0, ReflowChildFlags::Default);
     DoUpdateThumbPosition(thumbFrame,
                           nsSize(aDesiredSize.Width(), aDesiredSize.Height()));
   }
@@ -428,12 +292,13 @@ void nsRangeFrame::ReflowAnonymousContent(nsPresContext* aPresContext,
     nsReflowStatus frameStatus;
     ReflowOutput progressDesiredSize(aReflowInput);
     ReflowChild(rangeProgressFrame, aPresContext, progressDesiredSize,
-                progressReflowInput, 0, 0, 0, frameStatus);
+                progressReflowInput, 0, 0, ReflowChildFlags::Default,
+                frameStatus);
     MOZ_ASSERT(
         frameStatus.IsFullyComplete(),
         "We gave our child unconstrained height, so it should be complete");
     FinishReflowChild(rangeProgressFrame, aPresContext, progressDesiredSize,
-                      &progressReflowInput, 0, 0, 0);
+                      &progressReflowInput, 0, 0, ReflowChildFlags::Default);
     DoUpdateRangeProgressFrame(
         rangeProgressFrame,
         nsSize(aDesiredSize.Width(), aDesiredSize.Height()));
@@ -446,10 +311,8 @@ a11y::AccType nsRangeFrame::AccessibleType() { return a11y::eHTMLRangeType; }
 
 double nsRangeFrame::GetValueAsFractionOfRange() {
   MOZ_ASSERT(mContent->IsHTMLElement(nsGkAtoms::input), "bad cast");
-  dom::HTMLInputElement* input =
-      static_cast<dom::HTMLInputElement*>(GetContent());
-
-  MOZ_ASSERT(input->ControlType() == NS_FORM_INPUT_RANGE);
+  auto* input = static_cast<dom::HTMLInputElement*>(GetContent());
+  MOZ_ASSERT(input->ControlType() == FormControlType::InputRange);
 
   Decimal value = input->GetValueAsDecimal();
   Decimal minimum = input->GetMinimum();
@@ -480,7 +343,7 @@ Decimal nsRangeFrame::GetValueAtEventPoint(WidgetGUIEvent* aEvent) {
   dom::HTMLInputElement* input =
       static_cast<dom::HTMLInputElement*>(GetContent());
 
-  MOZ_ASSERT(input->ControlType() == NS_FORM_INPUT_RANGE);
+  MOZ_ASSERT(input->ControlType() == FormControlType::InputRange);
 
   Decimal minimum = input->GetMinimum();
   Decimal maximum = input->GetMaximum();
@@ -499,8 +362,8 @@ Decimal nsRangeFrame::GetValueAtEventPoint(WidgetGUIEvent* aEvent) {
   } else {
     absPoint = aEvent->mRefPoint;
   }
-  nsPoint point =
-      nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent, absPoint, this);
+  nsPoint point = nsLayoutUtils::GetEventCoordinatesRelativeTo(
+      aEvent, absPoint, RelativeTo{this});
 
   if (point == nsPoint(NS_UNCONSTRAINEDSIZE, NS_UNCONSTRAINEDSIZE)) {
     // We don't want to change the current value for this error state.
@@ -516,9 +379,9 @@ Decimal nsRangeFrame::GetValueAtEventPoint(WidgetGUIEvent* aEvent) {
     nsPresContext* presContext = PresContext();
     bool notUsedCanOverride;
     LayoutDeviceIntSize size;
-    presContext->GetTheme()->GetMinimumWidgetSize(presContext, this,
-                                                  StyleAppearance::RangeThumb,
-                                                  &size, &notUsedCanOverride);
+    presContext->Theme()->GetMinimumWidgetSize(presContext, this,
+                                               StyleAppearance::RangeThumb,
+                                               &size, &notUsedCanOverride);
     thumbSize.width = presContext->DevPixelsToAppUnits(size.width);
     thumbSize.height = presContext->DevPixelsToAppUnits(size.height);
     // For GTK, GetMinimumWidgetSize returns zero for the thumb dimension
@@ -568,7 +431,7 @@ Decimal nsRangeFrame::GetValueAtEventPoint(WidgetGUIEvent* aEvent) {
 }
 
 void nsRangeFrame::UpdateForValueChange() {
-  if (NS_SUBTREE_DIRTY(this)) {
+  if (IsSubtreeDirty()) {
     return;  // we're going to be updated when we reflow
   }
   nsIFrame* rangeProgressFrame = mProgressDiv->GetPrimaryFrame();
@@ -704,7 +567,7 @@ nsresult nsRangeFrame::AttributeChanged(int32_t aNameSpaceID,
       MOZ_ASSERT(mContent->IsHTMLElement(nsGkAtoms::input), "bad cast");
       bool typeIsRange =
           static_cast<dom::HTMLInputElement*>(GetContent())->ControlType() ==
-          NS_FORM_INPUT_RANGE;
+          FormControlType::InputRange;
       // If script changed the <input>'s type before setting these attributes
       // then we don't need to do anything since we are going to be reframed.
       if (typeIsRange) {
@@ -719,36 +582,41 @@ nsresult nsRangeFrame::AttributeChanged(int32_t aNameSpaceID,
   return nsContainerFrame::AttributeChanged(aNameSpaceID, aAttribute, aModType);
 }
 
-nscoord nsRangeFrame::AutoCrossSize(nscoord aEm) {
+nscoord nsRangeFrame::AutoCrossSize(Length aEm) {
   nscoord minCrossSize(0);
   if (IsThemed()) {
     bool unused;
     LayoutDeviceIntSize size;
     nsPresContext* pc = PresContext();
-    pc->GetTheme()->GetMinimumWidgetSize(pc, this, StyleAppearance::RangeThumb,
-                                         &size, &unused);
+    pc->Theme()->GetMinimumWidgetSize(pc, this, StyleAppearance::RangeThumb,
+                                      &size, &unused);
     minCrossSize =
         pc->DevPixelsToAppUnits(IsHorizontal() ? size.height : size.width);
   }
-  return std::max(minCrossSize, NSToCoordRound(CROSS_AXIS_EM_SIZE * aEm));
+  return std::max(minCrossSize, aEm.ScaledBy(CROSS_AXIS_EM_SIZE).ToAppUnits());
+}
+
+static mozilla::Length OneEm(nsRangeFrame* aFrame) {
+  return aFrame->StyleFont()->mFont.size.ScaledBy(
+      nsLayoutUtils::FontSizeInflationFor(aFrame));
 }
 
 LogicalSize nsRangeFrame::ComputeAutoSize(
     gfxContext* aRenderingContext, WritingMode aWM, const LogicalSize& aCBSize,
     nscoord aAvailableISize, const LogicalSize& aMargin,
-    const LogicalSize& aBorder, const LogicalSize& aPadding,
+    const LogicalSize& aBorderPadding, const StyleSizeOverrides& aSizeOverrides,
     ComputeSizeFlags aFlags) {
   bool isInlineOriented = IsInlineOriented();
-  auto em = StyleFont()->mFont.size * nsLayoutUtils::FontSizeInflationFor(this);
+  auto em = OneEm(this);
 
   const WritingMode wm = GetWritingMode();
   LogicalSize autoSize(wm);
   if (isInlineOriented) {
-    autoSize.ISize(wm) = NSToCoordRound(MAIN_AXIS_EM_SIZE * em);
+    autoSize.ISize(wm) = em.ScaledBy(MAIN_AXIS_EM_SIZE).ToAppUnits();
     autoSize.BSize(wm) = AutoCrossSize(em);
   } else {
     autoSize.ISize(wm) = AutoCrossSize(em);
-    autoSize.BSize(wm) = NSToCoordRound(MAIN_AXIS_EM_SIZE * em);
+    autoSize.BSize(wm) = em.ScaledBy(MAIN_AXIS_EM_SIZE).ToAppUnits();
   }
 
   return autoSize.ConvertTo(aWM, wm);
@@ -767,9 +635,11 @@ nscoord nsRangeFrame::GetMinISize(gfxContext* aRenderingContext) {
 }
 
 nscoord nsRangeFrame::GetPrefISize(gfxContext* aRenderingContext) {
-  bool isInline = IsInlineOriented();
-  auto em = StyleFont()->mFont.size * nsLayoutUtils::FontSizeInflationFor(this);
-  return isInline ? NSToCoordRound(em * MAIN_AXIS_EM_SIZE) : AutoCrossSize(em);
+  auto em = OneEm(this);
+  if (IsInlineOriented()) {
+    return em.ScaledBy(MAIN_AXIS_EM_SIZE).ToAppUnits();
+  }
+  return AutoCrossSize(em);
 }
 
 bool nsRangeFrame::IsHorizontal() const {
@@ -802,42 +672,16 @@ double nsRangeFrame::GetValue() const {
       .toDouble();
 }
 
-#define STYLES_DISABLING_NATIVE_THEMING                          \
-  NS_AUTHOR_SPECIFIED_BACKGROUND | NS_AUTHOR_SPECIFIED_PADDING | \
-      NS_AUTHOR_SPECIFIED_BORDER
-
 bool nsRangeFrame::ShouldUseNativeStyle() const {
   nsIFrame* trackFrame = mTrackDiv->GetPrimaryFrame();
   nsIFrame* progressFrame = mProgressDiv->GetPrimaryFrame();
   nsIFrame* thumbFrame = mThumbDiv->GetPrimaryFrame();
 
-  return (StyleDisplay()->mAppearance == StyleAppearance::Range) &&
+  return StyleDisplay()->EffectiveAppearance() == StyleAppearance::Range &&
          trackFrame &&
-         !PresContext()->HasAuthorSpecifiedRules(
-             trackFrame, STYLES_DISABLING_NATIVE_THEMING) &&
+         !trackFrame->Style()->HasAuthorSpecifiedBorderOrBackground() &&
          progressFrame &&
-         !PresContext()->HasAuthorSpecifiedRules(
-             progressFrame, STYLES_DISABLING_NATIVE_THEMING) &&
+         !progressFrame->Style()->HasAuthorSpecifiedBorderOrBackground() &&
          thumbFrame &&
-         !PresContext()->HasAuthorSpecifiedRules(
-             thumbFrame, STYLES_DISABLING_NATIVE_THEMING);
-}
-
-ComputedStyle* nsRangeFrame::GetAdditionalComputedStyle(int32_t aIndex) const {
-  // We only implement this so that SetAdditionalComputedStyle will be
-  // called if style changes that would change the -moz-focus-outer
-  // pseudo-element have occurred.
-  if (aIndex != 0) {
-    return nullptr;
-  }
-  return mOuterFocusStyle;
-}
-
-void nsRangeFrame::SetAdditionalComputedStyle(int32_t aIndex,
-                                              ComputedStyle* aComputedStyle) {
-  MOZ_ASSERT(aIndex == 0,
-             "GetAdditionalComputedStyle is handling other indexes?");
-
-  // The -moz-focus-outer pseudo-element's style has changed.
-  mOuterFocusStyle = aComputedStyle;
+         !thumbFrame->Style()->HasAuthorSpecifiedBorderOrBackground();
 }
