@@ -57,12 +57,11 @@ class nsILineIterator {
      */
     nsRect mLineBounds;
     /** Whether the line is wrapped at the end */
-    bool mIsWrapped;
+    bool mIsWrapped = false;
   };
 
   // Return miscellaneous information about a line.
-  virtual mozilla::Result<LineInfo, nsresult> GetLine(
-      int32_t aLineNumber) const = 0;
+  virtual mozilla::Result<LineInfo, nsresult> GetLine(int32_t aLineNumber) = 0;
 
   /**
    * Given a frame that's a child of the block, find which line its on
@@ -80,12 +79,7 @@ class nsILineIterator {
   // appropriately.
   NS_IMETHOD FindFrameAt(int32_t aLineNumber, nsPoint aPos,
                          nsIFrame** aFrameFound, bool* aPosIsBeforeFirstFrame,
-                         bool* aPosIsAfterLastFrame) const = 0;
-
-  // Give the line iterator implementor a chance todo something more complicated
-  // than nsIFrame::GetNextSibling()
-  NS_IMETHOD GetNextSiblingOnLine(nsIFrame*& aFrame,
-                                  int32_t aLineNumber) const = 0;
+                         bool* aPosIsAfterLastFrame) = 0;
 
   // Check whether visual and logical order of frames within a line are
   // identical.
