@@ -124,13 +124,6 @@ class nsCSSProps {
       kSubpropertyTable[eCSSProperty_COUNT - eCSSProperty_COUNT_no_shorthands];
 
  public:
-  /**
-   * Returns true if the backdrop-filter pref and WebRender are enabled.
-   */
-  static bool IsBackdropFilterAvailable(JSContext*, JSObject*) {
-    return IsEnabled(eCSSProperty_backdrop_filter);
-  }
-
   /**
    * Recoumputes the enabled state of a pref. If aPrefName is nullptr,
    * recomputes the state of all prefs in gPropertyEnabled.
