@@ -28,6 +28,9 @@
 #include "mozilla/GeckoBindings.h"
 #include "PreferenceSheet.h"
 #include "nsGlobalWindowOuter.h"
+#ifdef XP_WIN
+#  include "mozilla/WindowsVersion.h"
+#endif
 
 using namespace mozilla;
 using mozilla::dom::DisplayMode;
@@ -98,6 +101,11 @@ bool Gecko_MediaFeatures_ShouldAvoidNativeTheme(const Document* aDocument) {
   return aDocument->ShouldAvoidNativeTheme();
 }
 
+bool Gecko_MediaFeatures_UseOverlayScrollbars(const Document* aDocument) {
+  nsPresContext* pc = aDocument->GetPresContext();
+  return pc && pc->UseOverlayScrollbars();
+}
+
 static nsDeviceContext* GetDeviceContextFor(const Document* aDocument) {
   nsPresContext* pc = aDocument->GetPresContext();
   if (!pc) {
@@ -149,7 +157,7 @@ uint32_t Gecko_MediaFeatures_GetColorDepth(const Document* aDocument) {
 
   if (!nsContentUtils::ShouldResistFingerprinting(aDocument)) {
     if (nsDeviceContext* dx = GetDeviceContextFor(aDocument)) {
-      dx->GetDepth(depth);
+      depth = dx->GetDepth();
     }
   }
 
@@ -218,30 +226,36 @@ StyleDisplayMode Gecko_MediaFeatures_GetDisplayMode(const Document* aDocument) {
   return static_cast<StyleDisplayMode>(browsingContext->DisplayMode());
 }
 
-nsAtom* Gecko_MediaFeatures_GetOperatingSystemVersion(
-    const Document* aDocument) {
-  using OperatingSystemVersion = LookAndFeel::OperatingSystemVersion;
-
-  if (nsContentUtils::ShouldResistFingerprinting(aDocument)) {
-    return nullptr;
-  }
-
-  int32_t metricResult;
-  if (NS_FAILED(LookAndFeel::GetInt(
-          LookAndFeel::IntID::OperatingSystemVersionIdentifier,
-          &metricResult))) {
-    return nullptr;
-  }
-
-  switch (OperatingSystemVersion(metricResult)) {
-    case OperatingSystemVersion::Windows7:
-      return nsGkAtoms::windows_win7;
-    case OperatingSystemVersion::Windows8:
-      return nsGkAtoms::windows_win8;
-    case OperatingSystemVersion::Windows10:
-      return nsGkAtoms::windows_win10;
+bool Gecko_MediaFeatures_MatchesPlatform(StylePlatform aPlatform) {
+  switch (aPlatform) {
+#if defined(XP_WIN)
+    case StylePlatform::Windows:
+      return true;
+    case StylePlatform::WindowsWin10:
+    case StylePlatform::WindowsWin7:
+    case StylePlatform::WindowsWin8: {
+      if (IsWin10OrLater()) {
+        return aPlatform == StylePlatform::WindowsWin10;
+      }
+      if (IsWin8OrLater()) {
+        return aPlatform == StylePlatform::WindowsWin8;
+      }
+      return aPlatform == StylePlatform::WindowsWin7;
+    }
+#elif defined(ANDROID)
+    case StylePlatform::Android:
+      return true;
+#elif defined(MOZ_WIDGET_GTK)
+    case StylePlatform::Linux:
+      return true;
+#elif defined(XP_MACOSX)
+    case StylePlatform::Macos:
+      return true;
+#else
+#  error "Unknown platform?"
+#endif
     default:
-      return nullptr;
+      return false;
   }
 }
 
@@ -253,21 +267,65 @@ bool Gecko_MediaFeatures_PrefersReducedMotion(const Document* aDocument) {
 }
 
 StylePrefersColorScheme Gecko_MediaFeatures_PrefersColorScheme(
-    const Document* aDocument) {
-  return aDocument->PrefersColorScheme();
+    const Document* aDocument, bool aUseContent) {
+  auto scheme = aUseContent ? LookAndFeel::PreferredColorSchemeForContent()
+                            : aDocument->PreferredColorScheme();
+  return scheme == ColorScheme::Dark ? StylePrefersColorScheme::Dark
+                                     : StylePrefersColorScheme::Light;
 }
 
+// Neither Linux, Windows, nor Mac have a way to indicate that low contrast is
+// preferred so we use the presence of an accessibility theme or forced colors
+// as a signal.
 StylePrefersContrast Gecko_MediaFeatures_PrefersContrast(
     const Document* aDocument) {
   if (nsContentUtils::ShouldResistFingerprinting(aDocument)) {
     return StylePrefersContrast::NoPreference;
   }
-  // Neither Linux, Windows, nor Mac have a way to indicate that low contrast is
-  // preferred so we use the presence of an accessibility theme as a signal.
-  if (!!LookAndFeel::GetInt(LookAndFeel::IntID::UseAccessibilityTheme, 0)) {
+  const auto& prefs = PreferenceSheet::PrefsFor(*aDocument);
+  if (!prefs.mUseAccessibilityTheme && prefs.mUseDocumentColors) {
+    return StylePrefersContrast::NoPreference;
+  }
+  const auto& colors = prefs.ColorsFor(ColorScheme::Light);
+  float ratio = RelativeLuminanceUtils::ContrastRatio(colors.mDefaultBackground,
+                                                      colors.mDefault);
+  // https://www.w3.org/TR/WCAG21/#contrast-minimum
+  if (ratio < 4.5f) {
+    return StylePrefersContrast::Less;
+  }
+  // https://www.w3.org/TR/WCAG21/#contrast-enhanced
+  if (ratio >= 7.0f) {
     return StylePrefersContrast::More;
   }
-  return StylePrefersContrast::NoPreference;
+  return StylePrefersContrast::Custom;
+}
+
+StyleDynamicRange Gecko_MediaFeatures_DynamicRange(const Document* aDocument) {
+  // Bug 1759772: Once HDR color is available, update each platform
+  // LookAndFeel implementation to return StyleDynamicRange::High when
+  // appropriate.
+  return StyleDynamicRange::Standard;
+}
+
+StyleDynamicRange Gecko_MediaFeatures_VideoDynamicRange(
+    const Document* aDocument) {
+  if (nsContentUtils::ShouldResistFingerprinting(aDocument)) {
+    return StyleDynamicRange::Standard;
+  }
+  // video-dynamic-range: high has 3 requirements:
+  // 1) high peak brightness
+  // 2) high contrast ratio
+  // 3) color depth > 24
+  // We check the color depth requirement before asking the LookAndFeel
+  // if it is HDR capable.
+  if (nsDeviceContext* dx = GetDeviceContextFor(aDocument)) {
+    if (dx->GetDepth() > 24 &&
+        LookAndFeel::GetInt(LookAndFeel::IntID::VideoDynamicRange)) {
+      return StyleDynamicRange::High;
+    }
+  }
+
+  return StyleDynamicRange::Standard;
 }
 
 static PointerCapabilities GetPointerCapabilities(const Document* aDocument,
