@@ -16,15 +16,18 @@ const IO_SERVICE_CONTRACTID = "@mozilla.org/network/io-service;1"
 // "<!--CLEAR-->"
 const BLANK_URL_FOR_CLEARING = "data:text/html;charset=UTF-8,%3C%21%2D%2DCLEAR%2D%2D%3E";
 
-Cu.import("resource://gre/modules/Timer.jsm");
-Cu.import("resource://reftest/AsyncSpellCheckTestHelper.jsm");
-Cu.import("resource://gre/modules/Services.jsm");
+const { setTimeout, clearTimeout } = ChromeUtils.import(
+    "resource://gre/modules/Timer.jsm"
+);
+const { onSpellCheck } = ChromeUtils.import(
+    "resource://reftest/AsyncSpellCheckTestHelper.jsm"
+);
+const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 // This will load chrome Custom Elements inside chrome documents:
-ChromeUtils.import("resource://gre/modules/CustomElementsListener.jsm", null);
+ChromeUtils.import("resource://gre/modules/CustomElementsListener.jsm");
 
 var gBrowserIsRemote;
-var gIsWebRenderEnabled;
 var gHaveCanvasSnapshot = false;
 var gCurrentURL;
 var gCurrentURLRecordResults;
@@ -198,7 +201,7 @@ function doPrintMode(contentRootElement) {
 function setupPrintMode() {
     var PSSVC =
         Cc[PRINTSETTINGS_CONTRACTID].getService(Ci.nsIPrintSettingsService);
-    var ps = PSSVC.newPrintSettings;
+    var ps = PSSVC.createNewPrintSettings();
     ps.paperWidth = 5;
     ps.paperHeight = 3;
 
@@ -268,7 +271,7 @@ function setupViewport(contentRootElement) {
 
     // XXX support viewconfig when needed
 }
- 
+
 
 function setupDisplayport(contentRootElement) {
     let promise = content.windowGlobalChild.getActor("ReftestFission").SetupDisplayportRoot();
@@ -419,11 +422,14 @@ const STATE_WAITING_FOR_APZ_FLUSH = 3;
 const STATE_WAITING_TO_FINISH = 4;
 const STATE_COMPLETED = 5;
 
-function FlushRendering(aFlushMode) {
+async function FlushRendering(aFlushMode) {
     let browsingContext = content.docShell.browsingContext;
     let ignoreThrottledAnimations = (aFlushMode === FlushMode.IGNORE_THROTTLED_ANIMATIONS);
-    let promise = content.windowGlobalChild.getActor("ReftestFission").sendQuery("FlushRendering", {browsingContext, ignoreThrottledAnimations});
-    return promise.then(function(result) {
+    // Ensure the refresh driver ticks at least once, this ensures some
+    // preference changes take effect.
+    let needsAnimationFrame = IsSnapshottableTestType();
+    try {
+        let result = await content.windowGlobalChild.getActor("ReftestFission").sendQuery("FlushRendering", {browsingContext, ignoreThrottledAnimations, needsAnimationFrame});
         for (let errorString of result.errorStrings) {
             LogError(errorString);
         }
@@ -433,11 +439,11 @@ function FlushRendering(aFlushMode) {
         for (let infoString of result.infoStrings) {
             LogInfo(infoString);
         }
-    }, function(reason) {
+    } catch (reason) {
         // We expect actors to go away causing sendQuery's to fail, so
         // just note it.
         LogInfo("FlushRendering sendQuery to parent rejected: " + reason);
-    });
+    }
 }
 
 function WaitForTestEnd(contentRootElement, inPrintMode, spellCheckedElements, forURL) {
@@ -862,7 +868,6 @@ function WaitForTestEnd(contentRootElement, inPrintMode, spellCheckedElements, f
                     }
                 }
               }
-              CheckLayerAssertions(contentRootElement);
             }
 
             if (!IsSnapshottableTestType()) {
@@ -949,7 +954,7 @@ async function OnDocumentLoad(uri)
         // Ignore load events for previous documents.
         return;
     }
-    
+
     var currentDoc = content && content.document;
 
     // Collect all editable, spell-checked elements.  It may be the case that
@@ -997,7 +1002,6 @@ async function OnDocumentLoad(uri)
             // Go into reftest-wait mode belatedly.
             WaitForTestEnd(contentRootElement, inPrintMode, [], uri);
         } else {
-            CheckLayerAssertions(contentRootElement);
             CheckForProcessCrashExpectation(contentRootElement);
             RecordResult(uri);
         }
@@ -1027,63 +1031,6 @@ async function OnDocumentLoad(uri)
     }
 }
 
-function CheckLayerAssertions(contentRootElement)
-{
-    if (!contentRootElement) {
-        return;
-    }
-    if (gIsWebRenderEnabled) {
-        // WebRender doesn't use layers, so let's not try checking layers
-        // assertions.
-        return;
-    }
-
-    var opaqueLayerElements = getOpaqueLayerElements(contentRootElement);
-    for (var i = 0; i < opaqueLayerElements.length; ++i) {
-        var elem = opaqueLayerElements[i];
-        try {
-            if (!windowUtils().isPartOfOpaqueLayer(elem)) {
-                SendFailedOpaqueLayer(elementDescription(elem) + ' is not part of an opaque layer');
-            }
-        } catch (e) {
-            SendFailedOpaqueLayer('got an exception while checking whether ' + elementDescription(elem) + ' is part of an opaque layer');
-        }
-    }
-    var layerNameToElementsMap = getAssignedLayerMap(contentRootElement);
-    var oneOfEach = [];
-    // Check that elements with the same reftest-assigned-layer share the same PaintedLayer.
-    for (var layerName in layerNameToElementsMap) {
-        try {
-            var elements = layerNameToElementsMap[layerName];
-            oneOfEach.push(elements[0]);
-            var numberOfLayers = windowUtils().numberOfAssignedPaintedLayers(elements);
-            if (numberOfLayers !== 1) {
-                SendFailedAssignedLayer('these elements are assigned to ' + numberOfLayers +
-                                        ' different layers, instead of sharing just one layer: ' +
-                                        elements.map(elementDescription).join(', '));
-            }
-        } catch (e) {
-            SendFailedAssignedLayer('got an exception while checking whether these elements share a layer: ' +
-                                    elements.map(elementDescription).join(', '));
-        }
-    }
-    // Check that elements with different reftest-assigned-layer are assigned to different PaintedLayers.
-    if (oneOfEach.length > 0) {
-        try {
-            var numberOfLayers = windowUtils().numberOfAssignedPaintedLayers(oneOfEach);
-            if (numberOfLayers !== oneOfEach.length) {
-                SendFailedAssignedLayer('these elements are assigned to ' + numberOfLayers +
-                                        ' different layers, instead of having none in common (expected ' +
-                                        oneOfEach.length + ' different layers): ' +
-                                        oneOfEach.map(elementDescription).join(', '));
-            }
-        } catch (e) {
-            SendFailedAssignedLayer('got an exception while checking whether these elements are assigned to different layers: ' +
-                                    oneOfEach.map(elementDescription).join(', '));
-        }
-    }
-}
-
 function CheckForProcessCrashExpectation(contentRootElement)
 {
     if (contentRootElement &&
@@ -1378,14 +1325,6 @@ function SendContentReady()
 
     let info = {};
 
-    // The webrender check has to be separate from the d2d checks
-    // since the d2d checks will throw an exception on non-windows platforms.
-    try {
-        gIsWebRenderEnabled = gfxInfo.WebRenderEnabled;
-    } catch (e) {
-        gIsWebRenderEnabled = false;
-    }
-
     try {
         info.D2DEnabled = gfxInfo.D2DEnabled;
         info.DWriteEnabled = gfxInfo.DWriteEnabled;
@@ -1460,7 +1399,7 @@ async function SendInitCanvasWithSnapshot(forURL)
         sendAsyncMessage("reftest:InitCanvasWithSnapshot");
 
         gHaveCanvasSnapshot = await promise;
-        return gHaveCanvasSnapshot; 
+        return gHaveCanvasSnapshot;
     }
 
     // For in-process browser, we have to make a synchronous request
@@ -1545,14 +1484,13 @@ async function SendUpdateCanvasForEvent(forURL, rectList, contentRootElement)
 
     var message;
 
-    if ((gIsWebRenderEnabled || shouldNotFlush(contentRootElement)) &&
-        !windowUtils().isMozAfterPaintPending) {
+    if (!windowUtils().isMozAfterPaintPending) {
         // Webrender doesn't have invalidation, and animations on the compositor
         // don't invoke any MozAfterEvent which means we have no invalidated
         // rect so we just invalidate the whole screen once we don't have
         // anymore paints pending. This will force the snapshot.
 
-        LogInfo("Webrender enabled, sending update whole canvas for invalidation");
+        LogInfo("Sending update whole canvas for invalidation");
         message = "reftest:UpdateWholeCanvasForInvalidation";
     } else {
         LogInfo("SendUpdateCanvasForEvent with " + rectList.length + " rects");
